@@ -156,6 +156,22 @@ extern char *poll_revents_to_str(const short revents);
 extern void send_fd_over_pipe(int socket, int fd);
 extern int receive_fd_over_pipe(int socket);
 
+/*
+ * Make full directory path.
+ *
+ * Will not error if directories already exist.
+ * Warning: directory creation is a not an atomic operation.
+ * This function iteratively builds the path until complete, or an error
+ * occurs.
+ *
+ * IN is_dir:
+ *   true: last path component is a directory, and should be created
+ *   false: last path component is a filename, do not create
+ *
+ * RET SLURM_SUCCESS or error.
+ */
+extern int mkdirpath(const char *pathname, mode_t mode, bool is_dir);
+
 /*
  * Recursively remove a directory and all contents.
  * Takes care not to follow any symlinks outside the target directory.
