@@ -41,7 +41,6 @@
 
 #define _GNU_SOURCE
 
-#include <assert.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include <grp.h>
@@ -55,6 +54,10 @@
 #include <sys/wait.h>
 #include <unistd.h>
 
+#ifdef WITH_SELINUX
+#include <selinux/selinux.h>
+#endif
+
 /* FIXME: Come up with a real solution for EUID instead of substituting RUID */
 #if defined(__NetBSD__)
 #define eaccess(p,m) (access((p),(m)))
@@ -64,19 +67,24 @@
 #include "slurm/slurm_errno.h"
 
 #include "src/common/env.h"
-#include "src/common/gres.h"
 #include "src/common/fd.h"
 #include "src/common/log.h"
-#include "src/common/plugstack.h"
-#include "src/common/slurm_mpi.h"
+#include "src/common/run_command.h"
+#include "src/common/spank.h"
 #include "src/common/strlcpy.h"
-#include "src/common/switch.h"
+#include "src/common/xmalloc.h"
 #include "src/common/xsignal.h"
 #include "src/common/xstring.h"
-#include "src/common/xmalloc.h"
-#include "src/slurmd/common/proctrack.h"
-#include "src/slurmd/common/task_plugin.h"
+
+#include "src/interfaces/auth.h"
+#include "src/interfaces/gres.h"
+#include "src/interfaces/mpi.h"
+#include "src/interfaces/proctrack.h"
+#include "src/interfaces/switch.h"
+#include "src/interfaces/task.h"
+
 #include "src/slurmd/slurmd/slurmd.h"
+#include "src/slurmd/slurmstepd/container.h"
 #include "src/slurmd/slurmstepd/pdebug.h"
 #include "src/slurmd/slurmstepd/task.h"
 #include "src/slurmd/slurmstepd/ulimits.h"
@@ -84,25 +92,24 @@
 /*
  * Static prototype definitions.
  */
-static void  _make_tmpdir(stepd_step_rec_t *job);
+static void  _make_tmpdir(stepd_step_rec_t *step);
 static int   _run_script_and_set_env(const char *name, const char *path,
-				     stepd_step_rec_t *job);
-static void  _proc_stdout(char *buf, stepd_step_rec_t *job);
-static char *_uint32_array_to_str(int array_len, const uint32_t *array);
+				     stepd_step_rec_t *step);
+static void  _proc_stdout(char *buf, stepd_step_rec_t *step);
 
 /*
  * Process TaskProlog output
  * "export NAME=value"	adds environment variables
  * "unset  NAME"	clears an environment variable
- * "print  <whatever>"	writes that to the job's stdout
+ * "print  <whatever>"	writes that to the step's stdout
  */
-static void _proc_stdout(char *buf, stepd_step_rec_t *job)
+static void _proc_stdout(char *buf, stepd_step_rec_t *step)
 {
 	bool end_buf = false;
 	int len;
 	char *buf_ptr, *name_ptr, *val_ptr;
 	char *end_line, *equal_ptr;
-	char ***env = &job->env;
+	char ***env = &step->env;
 
 	buf_ptr = buf;
 	while (buf_ptr[0]) {
@@ -130,10 +137,10 @@ static void _proc_stdout(char *buf, stepd_step_rec_t *job)
 			equal_ptr[0] = '\0';
 			end_line[0] = '\0';
 			if (!xstrcmp(name_ptr, "SLURM_PROLOG_CPU_MASK")) {
-				job->cpu_bind_type = CPU_BIND_MASK;
-				xfree(job->cpu_bind);
-				job->cpu_bind = xstrdup(val_ptr);
-				if (task_g_pre_launch(job)) {
+				step->cpu_bind_type = CPU_BIND_MASK;
+				xfree(step->cpu_bind);
+				step->cpu_bind = xstrdup(val_ptr);
+				if (task_g_pre_launch(step)) {
 					error("Failed SLURM_PROLOG_CPU_MASK "
 					      "setup");
 					exit(1);
@@ -180,85 +187,52 @@ rwfail:		 /* process rest of script output */
  *	in the script's standard output.
  * name IN: class of program ("system prolog", "user prolog", etc.)
  * path IN: pathname of program to run
- * job IN/OUT: pointer to associated job, can update job->env
+ * step IN/OUT: pointer to associated step, can update step->env
  *	if prolog
- * RET 0 on success, -1 on failure.
+ * RET the exit status of the script or 1 on generic error and 0 on success
  */
 static int
 _run_script_and_set_env(const char *name, const char *path,
-			stepd_step_rec_t *job)
+			stepd_step_rec_t *step)
 {
-	int status, rc;
-	pid_t cpid;
-	int pfd[2];
-	char buf[4096];
-	FILE *f;
+	int status = 0, rc = 0;
+	char *argv[2];
+	char *buf = NULL;
+	run_command_args_t args = {
+		.job_id = step->step_id.job_id,
+		.max_wait = -1,
+		.script_path = path,
+		.script_type = name,
+		.status = &status
+	};
 
-	xassert(job->env);
 	if (path == NULL || path[0] == '\0')
-		return 0;
+		return rc;
 
-	debug("[job %u] attempting to run %s [%s]", job->jobid, name, path);
+	xassert(step->env);
+	setenvf(&step->env, "SLURM_SCRIPT_CONTEXT", "prolog_task");
+	args.env = step->env;
 
-	if (access(path, R_OK | X_OK) < 0) {
-		error("Could not run %s [%s]: %m", name, path);
-		return -1;
-	}
-	if (pipe(pfd) < 0) {
-		error("executing %s: pipe: %m", name);
-		return -1;
-	}
-	if ((cpid = fork()) < 0) {
-		error("executing %s: fork: %m", name);
-		return -1;
-	}
-	if (cpid == 0) {
-		char *argv[2];
+	argv[0] = xstrdup(path);
+	argv[1] = NULL;
+	args.script_argv = argv;
 
-		setenvf(&job->env, "SLURM_SCRIPT_CONTEXT", "prolog_task");
+	debug("[job %u] attempting to run %s [%s]",
+	      step->step_id.job_id, name, path);
+	buf = run_command(&args);
 
-		argv[0] = xstrdup(path);
-		argv[1] = NULL;
-		if (dup2(pfd[1], 1) == -1)
-			error("couldn't do the dup: %m");
-		close(2);
-		close(0);
-		close(pfd[0]);
-		close(pfd[1]);
-		setpgid(0, 0);
-		execve(path, argv, job->env);
-		error("execve(%s): %m", path);
-		exit(127);
+	if (WIFEXITED(status)) {
+		if (buf)
+			_proc_stdout(buf, step);
+		rc = WEXITSTATUS(status);
+	} else {
+		error("%s did not exit normally. reason: %s", name, buf);
+		rc = 1;
 	}
 
-	close(pfd[1]);
-	f = fdopen(pfd[0], "r");
-	if (f == NULL) {
-		error("Cannot open pipe device: %m");
-		log_fini();
-		exit(1);
-	}
-	while (feof(f) == 0) {
-		if (fgets(buf, sizeof(buf) - 1, f) != NULL) {
-			_proc_stdout(buf, job);
-		}
-	}
-	fclose(f);
-
-	while (1) {
-		rc = waitpid(cpid, &status, 0);
-		if (rc < 0) {
-			if (errno == EINTR)
-				continue;
-			error("waidpid: %m");
-			return 0;
-		} else  {
-			killpg(cpid, SIGKILL);  /* kill children too */
-			return status;
-		}
-	}
-
-	/* NOTREACHED */
+	xfree(argv[0]);
+	xfree(buf);
+	return rc;
 }
 
 /* Given a program name, translate it to a fully qualified pathname as needed
@@ -266,8 +240,8 @@ _run_script_and_set_env(const char *name, const char *path,
  * Returns xmalloc()'d string that must be xfree()'d */
 static char *_build_path(char *fname, char **prog_env)
 {
-	char *path_env = NULL, *dir;
-	char *file_name;
+	char *path_env = NULL, *dir = NULL;
+	char *file_name, *last = NULL;
 	struct stat stat_buf;
 	int len = PATH_MAX;
 
@@ -294,14 +268,14 @@ static char *_build_path(char *fname, char **prog_env)
 
 	/* search for the file using PATH environment variable */
 	path_env = xstrdup(getenvp(prog_env, "PATH"));
-
-	dir = strtok(path_env, ":");
+	if (path_env)
+		dir = strtok_r(path_env, ":", &last);
 	while (dir) {
 		snprintf(file_name, len, "%s/%s", dir, fname);
 		if ((stat(file_name, &stat_buf) == 0)
 		    && (! S_ISDIR(stat_buf.st_mode)))
 			break;
-		dir = strtok(NULL, ":");
+		dir = strtok_r(NULL, ":", &last);
 	}
 	if (dir == NULL)	/* not found */
 		strlcpy(file_name, fname, len);
@@ -311,164 +285,195 @@ static char *_build_path(char *fname, char **prog_env)
 }
 
 static int
-_setup_mpi(stepd_step_rec_t *job, int ltaskid)
+_setup_mpi(stepd_step_rec_t *step, int ltaskid)
 {
-	mpi_plugin_task_info_t info[1];
+	mpi_task_info_t info[1];
 
-	if (job->het_job_id && (job->het_job_id != NO_VAL)) {
-		info->jobid   = job->het_job_id;
-		info->stepid  = job->stepid;
-		info->nnodes  = job->het_job_nnodes;
-		info->nodeid  = job->het_job_node_offset + job->nodeid;
-		info->ntasks  = job->het_job_ntasks;
-		info->ltasks  = job->node_tasks;
-		info->gtaskid = job->het_job_task_offset +
-				job->task[ltaskid]->gtid;
-		info->ltaskid = job->task[ltaskid]->id;
-		info->self    = job->envtp->self;
-		info->client  = job->envtp->cli;
+	if (step->het_job_id && (step->het_job_id != NO_VAL))
+		info->step_id.job_id   = step->het_job_id;
+	else
+		info->step_id.job_id   = step->step_id.job_id;
+
+	if (step->het_job_offset != NO_VAL) {
+		info->step_id.step_id  = step->step_id.step_id;
+		info->step_id.step_het_comp  = step->step_id.step_het_comp;
+		info->nnodes  = step->het_job_nnodes;
+		info->nodeid  = step->het_job_node_offset + step->nodeid;
+		info->ntasks  = step->het_job_ntasks;
+		info->ltasks  = step->node_tasks;
+		info->gtaskid = step->het_job_task_offset +
+				step->task[ltaskid]->gtid;
+		info->ltaskid = step->task[ltaskid]->id;
+		info->self    = step->envtp->self;
+		info->client  = step->envtp->cli;
 	} else {
-		info->jobid   = job->jobid;
-		info->stepid  = job->stepid;
-		info->nnodes  = job->nnodes;
-		info->nodeid  = job->nodeid;
-		info->ntasks  = job->ntasks;
-		info->ltasks  = job->node_tasks;
-		info->gtaskid = job->task[ltaskid]->gtid;
-		info->ltaskid = job->task[ltaskid]->id;
-		info->self    = job->envtp->self;
-		info->client  = job->envtp->cli;
+		info->step_id.step_id  = step->step_id.step_id;
+		info->step_id.step_het_comp  = step->step_id.step_het_comp;
+		info->nnodes  = step->nnodes;
+		info->nodeid  = step->nodeid;
+		info->ntasks  = step->ntasks;
+		info->ltasks  = step->node_tasks;
+		info->gtaskid = step->task[ltaskid]->gtid;
+		info->ltaskid = step->task[ltaskid]->id;
+		info->self    = step->envtp->self;
+		info->client  = step->envtp->cli;
 	}
 
-	return mpi_hook_slurmstepd_task(info, &job->env);
+	return mpi_g_slurmstepd_task(info, &step->env);
 }
 
 /*
  *  Current process is running as the user when this is called.
  */
-extern void exec_task(stepd_step_rec_t *job, int local_proc_id)
+extern void exec_task(stepd_step_rec_t *step, int local_proc_id)
 {
-	uint32_t *gtids;		/* pointer to array of ranks */
 	int fd, j;
-	stepd_step_task_info_t *task = job->task[local_proc_id];
+	stepd_step_task_info_t *task = step->task[local_proc_id];
 	char **tmp_env;
-	int saved_errno;
+	int saved_errno, status;
 	uint32_t node_offset = 0, task_offset = 0;
 
-	if (job->het_job_node_offset != NO_VAL)
-		node_offset = job->het_job_node_offset;
-	if (job->het_job_task_offset != NO_VAL)
-		task_offset = job->het_job_task_offset;
+	if (step->container)
+		container_task_init(step, task);
 
-	gtids = xmalloc(job->node_tasks * sizeof(uint32_t));
-	for (j = 0; j < job->node_tasks; j++)
-		gtids[j] = job->task[j]->gtid + task_offset;
-	job->envtp->sgtids = _uint32_array_to_str(job->node_tasks, gtids);
-	xfree(gtids);
+	if (step->het_job_node_offset != NO_VAL)
+		node_offset = step->het_job_node_offset;
+	if (step->het_job_task_offset != NO_VAL)
+		task_offset = step->het_job_task_offset;
 
-	if (job->het_job_id != NO_VAL)
-		job->envtp->jobid = job->het_job_id;
+	for (j = 0; j < step->node_tasks; j++)
+		xstrfmtcat(step->envtp->sgtids, "%s%u", j ? "," : "",
+			   step->task[j]->gtid + task_offset);
+
+	if (step->het_job_id != NO_VAL)
+		step->envtp->jobid = step->het_job_id;
 	else
-		job->envtp->jobid = job->jobid;
-	job->envtp->stepid = job->stepid;
-	job->envtp->nodeid = job->nodeid + node_offset;
-	job->envtp->cpus_on_node = job->cpus;
-	job->envtp->procid = task->gtid + task_offset;
-	job->envtp->localid = task->id;
-	job->envtp->task_pid = getpid();
-	job->envtp->distribution = job->task_dist;
-	job->envtp->cpu_bind = xstrdup(job->cpu_bind);
-	job->envtp->cpu_bind_type = job->cpu_bind_type;
-	job->envtp->cpu_freq_min = job->cpu_freq_min;
-	job->envtp->cpu_freq_max = job->cpu_freq_max;
-	job->envtp->cpu_freq_gov = job->cpu_freq_gov;
-	job->envtp->mem_bind = xstrdup(job->mem_bind);
-	job->envtp->mem_bind_type = job->mem_bind_type;
-	job->envtp->distribution = -1;
-	job->envtp->batch_flag = job->batch;
-	job->envtp->uid = job->uid;
-	job->envtp->user_name = xstrdup(job->user_name);
+		step->envtp->jobid = step->step_id.job_id;
+	step->envtp->stepid = step->step_id.step_id;
+	step->envtp->nodeid = step->nodeid + node_offset;
+	step->envtp->cpus_on_node = step->cpus;
+	step->envtp->procid = task->gtid + task_offset;
+	step->envtp->localid = task->id;
+	step->envtp->task_pid = getpid();
+	step->envtp->distribution = step->task_dist;
+	step->envtp->cpu_bind = xstrdup(step->cpu_bind);
+	step->envtp->cpu_bind_type = step->cpu_bind_type;
+	step->envtp->cpu_freq_min = step->cpu_freq_min;
+	step->envtp->cpu_freq_max = step->cpu_freq_max;
+	step->envtp->cpu_freq_gov = step->cpu_freq_gov;
+	step->envtp->mem_bind = xstrdup(step->mem_bind);
+	step->envtp->mem_bind_type = step->mem_bind_type;
+	step->envtp->distribution = -1;
+	step->envtp->batch_flag = step->batch;
+	step->envtp->uid = step->uid;
+	step->envtp->job_end_time = step->job_end_time;
+	step->envtp->job_licenses = xstrdup(step->job_licenses);
+	step->envtp->job_start_time = step->job_start_time;
+	step->envtp->user_name = xstrdup(step->user_name);
 
 	/*
-	 * Modify copy of job's environment. Do not alter in place or
+	 * Modify copy of step's environment. Do not alter in place or
 	 * concurrent searches of the environment can generate invalid memory
 	 * references.
 	 */
-	job->envtp->env = env_array_copy((const char **) job->env);
-	setup_env(job->envtp, false);
-	setenvf(&job->envtp->env, "SLURM_JOB_GID", "%d", job->gid);
-	setenvf(&job->envtp->env, "SLURMD_NODENAME", "%s", conf->node_name);
-	if (job->tres_bind) {
-		setenvf(&job->envtp->env, "SLURMD_TRES_BIND", "%s",
-			job->tres_bind);
+	step->envtp->env = env_array_copy((const char **) step->env);
+	setup_env(step->envtp, false);
+	setenvf(&step->envtp->env, "SLURM_JOB_GID", "%u", step->gid);
+	setenvf(&step->envtp->env, "SLURMD_NODENAME", "%s", conf->node_name);
+	if (step->tres_bind) {
+		setenvf(&step->envtp->env, "SLURMD_TRES_BIND", "%s",
+			step->tres_bind);
 	}
-	if (job->tres_freq) {
-		setenvf(&job->envtp->env, "SLURMD_TRES_FREQ", "%s",
-			job->tres_freq);
+	if (step->tres_freq) {
+		setenvf(&step->envtp->env, "SLURMD_TRES_FREQ", "%s",
+			step->tres_freq);
 	}
-	tmp_env = job->env;
-	job->env = job->envtp->env;
+	tmp_env = step->env;
+	step->env = step->envtp->env;
 	env_array_free(tmp_env);
-	job->envtp->env = NULL;
+	step->envtp->env = NULL;
 
-	xfree(job->envtp->task_count);
+	xfree(step->envtp->task_count);
 
-	if (!job->batch && (job->stepid != SLURM_EXTERN_CONT)) {
-		if (switch_g_job_attach(job->switch_job, &job->env,
-					job->nodeid, (uint32_t) local_proc_id,
-					job->nnodes, job->ntasks,
-					task->gtid) < 0) {
+	if (!step->batch && (step->step_id.step_id != SLURM_EXTERN_CONT) &&
+	    (step->step_id.step_id != SLURM_INTERACTIVE_STEP)) {
+		if (switch_g_job_attach(step->switch_job, &step->env,
+					step->nodeid, (uint32_t) local_proc_id,
+					step->nnodes, step->ntasks,
+					task->gtid + task_offset) < 0) {
 			error("Unable to attach to interconnect: %m");
 			log_fini();
-			exit(1);
+			_exit(1);
 		}
 
-		if (_setup_mpi(job, local_proc_id) != SLURM_SUCCESS) {
+		if (_setup_mpi(step, local_proc_id) != SLURM_SUCCESS) {
 			error("Unable to configure MPI plugin: %m");
 			log_fini();
-			exit(1);
+			_exit(1);
 		}
 	}
 
 	/* task-specific pre-launch activities */
 
 	/* task plugin hook */
-	if (task_g_pre_launch(job)) {
+	if (task_g_pre_launch(step)) {
 		error("Failed to invoke task plugins: task_p_pre_launch error");
-		exit(1);
+		_exit(1);
 	}
-	if (!job->batch && (job->accel_bind_type || job->tres_bind)) {
+	if (!step->batch && (step->step_id.step_id != SLURM_INTERACTIVE_STEP) &&
+	    (step->accel_bind_type || step->tres_bind)) {
 		/*
-		 * Modify copy of job's environment as needed for GRES. Do not
+		 * Modify copy of step's environment as needed for GRES. Do not
 		 * alter in place or concurrent searches of the environment can
 		 * generate invalid memory references.
 		 */
-		job->envtp->env = env_array_copy((const char **) job->env);
-		gres_plugin_step_set_env(&job->envtp->env, job->step_gres_list,
-					 job->accel_bind_type, job->tres_bind,
-					 local_proc_id);
-		tmp_env = job->env;
-		job->env = job->envtp->env;
+		step->envtp->env = env_array_copy((const char **) step->env);
+		gres_g_task_set_env(step, local_proc_id);
+		tmp_env = step->env;
+		step->env = step->envtp->env;
 		env_array_free(tmp_env);
 	}
 
-	if (spank_user_task(job, local_proc_id) < 0) {
+	/*
+	 * test7.21 calls slurm_load_job() as an example of weird things people
+	 * may do within a SPANK stack. That will deadlock if we don't drop the
+	 * lock here.
+	 */
+	auth_setuid_unlock();
+	if (spank_user_task(step, local_proc_id) < 0) {
 		error("Failed to invoke spank plugin stack");
-		exit(1);
+		_exit(1);
 	}
+	auth_setuid_lock();
 
-	if (conf->task_prolog) {
-		char *my_prolog;
-		slurm_mutex_lock(&conf->config_mutex);
-		my_prolog = xstrdup(conf->task_prolog);
-		slurm_mutex_unlock(&conf->config_mutex);
-		_run_script_and_set_env("slurm task_prolog",
-					my_prolog, job);
-		xfree(my_prolog);
+#ifdef WITH_SELINUX
+	if (setexeccon(step->selinux_context)) {
+		error("Failed to set SELinux context to %s: %m",
+		      step->selinux_context);
+		_exit(1);
 	}
-	if (job->task_prolog) {
-		_run_script_and_set_env("user task_prolog",
-					job->task_prolog, job);
+#else
+	if (step->selinux_context) {
+		error("Built without SELinux support but context was specified");
+		_exit(1);
+	}
+#endif
+
+	if (slurm_conf.task_prolog) {
+		status = _run_script_and_set_env("slurm task_prolog",
+						 slurm_conf.task_prolog, step);
+		if (status) {
+			error("TaskProlog failed status=%d", status);
+			_exit(status);
+		}
+	}
+	if (step->task_prolog) {
+		status = _run_script_and_set_env("user task_prolog",
+						 step->task_prolog, step);
+		if (status) {
+			error("--task-prolog failed status=%d", status);
+			_exit(status);
+		}
 	}
 
 	/*
@@ -476,19 +481,19 @@ extern void exec_task(stepd_step_rec_t *job, int local_proc_id)
 	 * might be set or changed in one of the prolog scripts.
 	 */
 	if (local_proc_id == 0)
-		_make_tmpdir(job);
+		_make_tmpdir(step);
 
-	if (!job->batch)
-		pdebug_stop_current(job);
-	if (job->env == NULL) {
-		debug("job->env is NULL");
-		job->env = (char **)xmalloc(sizeof(char *));
-		job->env[0] = (char *)NULL;
+	if (!step->batch)
+		pdebug_stop_current(step);
+	if (step->env == NULL) {
+		debug("step->env is NULL");
+		step->env = (char **)xmalloc(sizeof(char *));
+		step->env[0] = (char *)NULL;
 	}
 
 	if (task->argv[0] == NULL) {
 		error("No executable program specified for this task");
-		exit(2);
+		_exit(2);
 	}
 
 	if (*task->argv[0] != '/') {
@@ -499,20 +504,30 @@ extern void exec_task(stepd_step_rec_t *job, int local_proc_id)
 		 * filesystem namespaces into the final arrangement, which
 		 * may affect which executable we select.
 		 */
-		task->argv[0] = _build_path(task->argv[0], job->env);
+		task->argv[0] = _build_path(task->argv[0], step->env);
 	}
 
 
 	/* Do this last so you don't worry too much about the users
 	   limits including the slurmstepd in with it.
 	*/
-	if (set_user_limits(job) < 0) {
-		debug("Unable to set user limits");
-		log_fini();
-		exit(5);
+	set_user_limits(step, 0);
+
+	/*
+	 * If argv[0] ends with '/' it indicates that srun was called with
+	 * --bcast with destination dir instead of file name. So match the
+	 * convention used by _rpc_file_bcast().
+	 */
+	if (task->argv[0][strlen(task->argv[0]) - 1] == '/') {
+		xstrfmtcat(task->argv[0], "slurm_bcast_%u.%u_%s",
+			   step->step_id.job_id, step->step_id.step_id,
+			   step->node_name);
 	}
 
-	execve(task->argv[0], task->argv, job->env);
+	if (step->container)
+		container_run(step, task);
+
+	execve(task->argv[0], task->argv, step->env);
 	saved_errno = errno;
 
 	/*
@@ -530,21 +545,21 @@ extern void exec_task(stepd_step_rec_t *job, int local_proc_id)
 				eol[0] = '\0';
 			slurm_seterrno(saved_errno);
 			error("execve(): bad interpreter(%s): %m", buf+2);
-			exit(errno);
+			_exit(errno);
 		}
 	}
 	slurm_seterrno(saved_errno);
 	error("execve(): %s: %m", task->argv[0]);
-	exit(errno);
+	_exit(errno);
 }
 
 static void
-_make_tmpdir(stepd_step_rec_t *job)
+_make_tmpdir(stepd_step_rec_t *step)
 {
 	char *tmpdir;
 
-	if (!(tmpdir = getenvp(job->env, "TMPDIR")))
-		setenvf(&job->env, "TMPDIR", "/tmp"); /* task may want it set */
+	if (!(tmpdir = getenvp(step->env, "TMPDIR")))
+		setenvf(&step->env, "TMPDIR", "/tmp"); /* task may want it set */
 	else if (mkdir(tmpdir, 0700) < 0) {
 		struct stat st;
 		int mkdir_errno = errno;
@@ -574,34 +589,8 @@ _make_tmpdir(stepd_step_rec_t *job)
 			return;
 
 		error("Setting TMPDIR to /tmp");
-		setenvf(&job->env, "TMPDIR", "/tmp");
+		setenvf(&step->env, "TMPDIR", "/tmp");
 	}
 
 	return;
 }
-
-/*
- * Return a string representation of an array of uint32_t elements.
- * Each value in the array is printed in decimal notation and elements
- * are separated by a comma.
- *
- * Returns an xmalloc'ed string.  Free with xfree().
- */
-static char *_uint32_array_to_str(int array_len, const uint32_t *array)
-{
-	int i;
-	char *sep = ",";  /* seperator */
-	char *str = xstrdup("");
-
-	if (array == NULL)
-		return str;
-
-	for (i = 0; i < array_len; i++) {
-
-		if (i == array_len-1) /* last time through loop */
-			sep = "";
-		xstrfmtcat(str, "%u%s", array[i], sep);
-	}
-
-	return str;
-}
