@@ -846,6 +846,8 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 	slurm_opt_t *opt_local = &opt;
 	uint32_t node_offset = 0, het_job_nnodes = 0, step_id = NO_VAL;
 	uint32_t het_job_ntasks = 0, task_offset = 0;
+	bool update_het_nnodes = false;
+	uint32_t updated_het_nnodes;
 
 	job_step_create_response_msg_t *step_resp;
 	char *resv_ports = NULL;
@@ -879,8 +881,10 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 			het_job_ntasks += job->ntasks;
 		}
 
+		updated_het_nnodes = het_job_nnodes;
 		list_iterator_reset(job_iter);
 		while ((job = list_next(job_iter))) {
+			uint32_t old_nhosts = job->nhosts;
 			if (opt_list)
 				opt_local = list_next(opt_iter);
 			if (!opt_local)
@@ -918,6 +922,23 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 			}
 			node_offset += job->nhosts;
 			task_offset += job->ntasks;
+
+			/*
+			 * If packing nodes (CR_PACK_NODES, -mpack), the step
+			 * may have an updated layout. Need to update each
+			 * component's het_job_nnodes with the updated counts.
+			 */
+			if (job->nhosts < old_nhosts) {
+				update_het_nnodes = true;
+				updated_het_nnodes -= old_nhosts - job->nhosts;
+			}
+		}
+
+		if (update_het_nnodes) {
+			list_iterator_reset(job_iter);
+			while ((job = list_next(job_iter))) {
+				job->het_job_nnodes = updated_het_nnodes;
+			}
 		}
 
 		FREE_NULL_HOSTLIST(exclude_hl);
@@ -1170,7 +1191,7 @@ extern void create_srun_job(void **p_job, bool *got_alloc,
 	ListIterator opt_iter, resp_iter;
 	srun_job_t *job = NULL;
 	int i, max_list_offset, max_het_job_offset, het_job_offset = -1,
-		het_step_offset = 0;
+		het_step_offset = -1;
 	uint32_t my_job_id = 0, het_job_id = 0;
 	char *het_job_nodelist = NULL;
 	bool begin_error_logged = false;
@@ -1179,6 +1200,7 @@ extern void create_srun_job(void **p_job, bool *got_alloc,
 	bool network_error_logged = false;
 #endif
 	bool node_cnt_error_logged = false;
+	bool tres_license_error_logged = false;
 	bool x11_error_logged = false;
 
 	/*
@@ -1241,6 +1263,8 @@ extern void create_srun_job(void **p_job, bool *got_alloc,
 				max_het_job_offset = max_list_offset;
 				local_het_step = true;
 			}
+			if (list_count(opt_list) > 1)
+				het_step_offset = 0;
 		}
 		srun_job_list = list_create(NULL);
 		used_resp_list = list_create(NULL);
@@ -1300,6 +1324,15 @@ extern void create_srun_job(void **p_job, bool *got_alloc,
 							opt_local->min_nodes;
 					}
 				}
+				if (!tres_license_error_logged &&
+				    !slurm_option_set_by_env(
+					    opt_local,
+					    LONG_OPT_TRES_PER_TASK) &&
+				    xstrstr(opt_local->tres_per_task,
+					    "license")) {
+					warning("Ignoring --tres-per-task license specification because licenses can only be specified at job allocation time, not during step allocation.");
+					tres_license_error_logged = true;
+				}
 				if (srun_opt->core_spec_set &&
 				    !core_spec_error_logged) {
 					/*
